<?php

if (! defined('ABSPATH')) {
    exit;
}

if (! class_exists('Business_Manager_Client_Metaboxes', false)) :

/**
 * Business_Manager_Setup Class.
 */
class Business_Manager_Client_Metaboxes {
    /**
     * Post type.
     * @var string
     */
    public $post_type = 'bm-client';

    public $date_format = '';

    /**
     * Metabox prefix.
     *
     * @since 1.0.0
     */
    private $pre = '_bm_client_';

    public function __construct() {
        $this->date_format = business_manager_datepicker_format();
        $this->hooks();
    }

    /**
     * Hook in to actions & filters.
     *
     * @since 1.0.0
     */
    public function hooks() {
        add_action('cmb2_admin_init', [$this, 'metabox_details']);
        add_action('cmb2_admin_init', [$this, 'metabox_files']);
        add_action('cmb2_admin_init', [$this, 'metabox_notes']);

        add_action('cmb2_admin_init', [$this, 'init_custom_fields']);
        // add_action( 'cmb2_admin_init', array( $this, 'metabox_permissions' ) );
    }

    public function init_custom_fields() {
        require_once BUSINESSMANAGER_DIR.'includes/class-fields.php';
        Business_Manager_User_Field::init_users();
        Business_Manager_Notes_Field::init_notes();
    }

    public function metabox_details() {
        $id = $this->pre.'details_box';

        $box_options = [
            'id' => $id,
            'title' => '<span class="dashicons dashicons-id-alt"></span> '.__('Client Details', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            'priority' => 'high',
        ];

        // Setup meta box
        $box = new_cmb2_box($box_options);

        // setting tabs
        $tabs_setting = [
            'config' => $box_options,
            //		'layout' => 'vertical', // Default : horizontal
            'tabs' => [],
        ];

        $fields = [];

        $fields['main'][] = [
            'name' => __('Address', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'address',
            'type' => 'text',
            'classes' => 'bm-field-full',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'address2',
            'type' => 'text',
            'classes' => 'bm-field-full',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('City', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'city',
            'type' => 'text',
            'classes' => 'bm-field-quarter',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('State/Province', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'state_province',
            'type' => 'text',
            'classes' => 'bm-field-quarter',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('Zipcode', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'zipcode',
            'type' => 'text',
            'classes' => 'bm-field-quarter',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('Country', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'country',
            'type' => 'text',
            'classes' => 'bm-field-quarter',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('Tax ID', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'tax_id',
            'type' => 'text',
            'classes' => 'bm-field-half',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('Phone', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'phone',
            'type' => 'text',
            'classes' => 'bm-field-clear bm-field-half',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('Email', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'email',
            'type' => 'text',
            'classes' => 'bm-field-half',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('Website', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'website',
            'type' => 'text',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('Logo', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'logo',
            'type' => 'file',
            'classes' => 'bm-field-clear',
            'options' => [
                'url' => false,
            ],
            'text' => [
                'add_upload_file_text' => 'Add or Upload Logo',
            ],
            'query_args' => [
                'type' => [
                    'image/gif',
                    'image/jpeg',
                    'image/png',
                ],
            ],
            'preview_size' => 'thumbnail',
        ];

        $fields['main'][] = [
            'name' => __('Working Hours', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'working_hours',
            'type' => 'text',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('Working Days', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'working_days',
            'type' => 'text',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('Main Contact Name', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'contact_name',
            'type' => 'text',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['main'][] = [
            'name' => __('Main Contact Phone', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'contact_phone',
            'type' => 'text',
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields['staff'][] = [
            'name' => __('Staff Members', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'staff',
            'type' => 'users',
            'repeatable' => true,
            'classes' => 'bm-repeat',
            'date_format' => $this->date_format,
            'text' => [
                'add_row_text' => __('Add Staff Member', 'business-manager'),
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        $tabs_setting['tabs'][] = [
            'id' => 'main',
            'title' => __('Main', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'fields' => $fields['main'],
        ];

        $tabs_setting['tabs'][] = [
            'id' => 'staff',
            'title' => __('Staff', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'fields' => $fields['staff'],
        ];

        // set tabs
        $box->add_field([
            'id' => '__tabs',
            'type' => 'tabs',
            'tabs' => $tabs_setting,
        ]);
    }

    public function metabox_notes() {
        $id = $this->pre.'notes_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-welcome-write-blog"></span> '.__('Notes', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            'priority' => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => __('Client Notes', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'notes',
            'type' => 'notes',
            'classes' => 'bm-field-full',
            'repeatable' => true,
            'date_format' => $this->date_format,
            'options' => [
                'add_row_text' => __('Add Note', 'business-manager'),
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }

    public function metabox_files() {
        $id = $this->pre.'files_box';

        $box_options = [
            'id' => $id,
            'title' => '<span class="dashicons dashicons-portfolio"></span> '.__('Files & Documents', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            // 'priority'   => 'high',
        ];

        // Setup meta box
        $box = new_cmb2_box($box_options);

        // setting tabs
        $tabs_setting = [
            'config' => $box_options,
            //		'layout' => 'vertical', // Default : horizontal
            'tabs' => [],
        ];

        $fields['general'][] = [
            'name' => __('Files', 'business-manager'),
            'id' => $this->pre.'general_files',
            'type' => 'file_list',
            'options' => [
                'url' => false,
            ],
        ];

        $fields['contracts'][] = [
            'name' => __('Files', 'business-manager'),
            'id' => $this->pre.'contract_files',
            'desc' => __('Contract documents & files', 'business-manager'),
            'type' => 'file_list',
            'options' => [
                'url' => false,
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        $tabs_setting['tabs'][] = [
            'id' => 'general',
            'title' => __('General', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'fields' => $fields['general'],
        ];
        $tabs_setting['tabs'][] = [
            'id' => 'contracts',
            'title' => __('Contracts', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'fields' => $fields['contracts'],
        ];

        // set tabs
        $box->add_field([
            'id' => '__tabs',
            'type' => 'tabs',
            'tabs' => $tabs_setting,
        ]);
    }
}

endif;

return new Business_Manager_Client_Metaboxes();
