<?php

if (!defined('ABSPATH')) {
    exit;
}

function business_manager_employee_full_name($id = null, $name_break = false)
{
    if (!$id) {
        return;
    }
    $fname = get_post_meta($id, '_bm_employee_first_name', true);
    $lname = get_post_meta($id, '_bm_employee_last_name', true);
    $return = $fname.($name_break == true ? '<br>' : ' ').$lname;
    return $return;
}

function business_manager_employee_job_title($id = null)
{
    if (!$id) {
        return;
    }
    $return = get_post_meta($id, '_bm_employee_title', true);
    return $return;
}

function business_manager_employee_dob($id = null)
{
    if (!$id) {
        return;
    }
    $return = get_post_meta($id, '_bm_employee_dob', true);
    return $return;
}

function business_manager_employee_status($id = null)
{
    if (!$id) {
        return;
    }
    $return = get_post_meta($id, '_bm_employee_status', true);
    return $return;
}

function business_manager_employee_type($id = null)
{
    if (!$id) {
        return;
    }
    $return = get_post_meta($id, '_bm_employee_type', true);
    return $return;
}

function business_manager_employee_work_phone($id = null)
{
    if (!$id) {
        return;
    }
    $return = get_post_meta($id, '_bm_employee_work_phone', true);
    return $return;
}

function business_manager_employee_work_email($id = null)
{
    if (!$id) {
        return;
    }
    $return = get_post_meta($id, '_bm_employee_work_email', true);
    return $return;
}

function business_manager_employee_personal_email($id = null)
{
    if (!$id) {
        return;
    }
    $return = get_post_meta($id, '_bm_employee_email', true);
    return $return;
}

function business_manager_employee_address($id = null)
{
    if (!$id) {
        return;
    }
    $return = get_post_meta($id, '_bm_employee_address', true);
    return $return;
}

function business_manager_employee_main_department($id = null)
{
    if (!$id) {
        return;
    }
    $terms = wp_get_object_terms($id, 'bm-department');
    if (!$terms) {
        return;
    }
    foreach ($terms as $key => $term) {
        if ($term->parent == 0) {
            $main_department = $term;
            break;
        } else {
            $main_department = $term;
        }
    }
    return $main_department;
}

function business_manager_employee_main_department_name($id = null)
{
    $department = business_manager_employee_main_department($id);
    return $department->name;
}

function business_manager_employee_photo($id = null)
{
    $photo = plugin_dir_url(dirname(__FILE__)).'/assets/images/placeholders/employee-photo.png';

    if ($id) {
        $image = wp_get_attachment_image_src($id, 'medium');
        if (is_array($image) && strlen($image[0]) > 0) {
            $photo = $image[0];
        }
    }

    $return = '<div class="bm-image-circle" style="background-image:url('.$photo.');">&nbsp;</div>';

    return $return;
}

function business_manager_employee_access($user_id, $access_type)
{
    if (current_user_can('bm_employee')) {
        $employee_id = business_manager_employee_id($user_id);

        return (get_post_meta($employee_id, $access_type, true) ?: 'none');
    }
}

function business_manager_employee_id($user_id)
{
    global $wpdb;

    if (current_user_can('bm_employee')) {
        $sql_employee = $wpdb->prepare(
            'SELECT post_id 
            FROM '.$wpdb->prefix."postmeta 
            WHERE meta_key = '_bm_employee_user_id' 
            AND meta_value = %d
            LIMIT 1",
            $user_id
        );
        $employee = $wpdb->get_col($sql_employee, 0);

        return ($employee[0] ?: 0);
    } else {
        return 0;
    }
}

function business_manager_client_logo($id = null)
{
    $logo = plugin_dir_url(dirname(__FILE__)).'/assets/images/placeholders/client-logo.png';

    if ($id) {
        $image = wp_get_attachment_image_src($id, 'medium');
        if (is_array($image) && strlen($image[0]) > 0) {
            $logo = $image[0];
        }
    }

    $return = '<div class="bm-image-circle" style="background-image:url('.$logo.');">&nbsp;</div>';

    return $return;
}

/**
 * Gets a post that is linked to another post.
 * ie. get leave data associated with an employee.
 *
 * @param $data Array the query parameters
 * @return Array the id and the post meta
 */
function business_manager_post_items($query_args = [])
{
    $args = wp_parse_args($query_args, [
        'numberposts' => -1,
        'post_status' => 'publish',
    ]);

    $posts = get_posts($args);

    $data = [];
    if ($posts) {
        $i = 0;
        foreach ($posts as $post) {
            $data[$i]['id'] = $post->ID;
            $data[$i]['meta'] = get_post_meta($post->ID, '', true);
            $i++;
        }
    }

    return $data;
}

/**
 * Gets number of posts for a post type.
 *
 * @param $post_type string the post type
 * @return string the formatted count of the post type
 */
function business_manager_item_count_text($post_type = '')
{
    if (!$post_type) {
        return;
    }

    $user = get_current_user_id();
    $user_bm_employee = get_user_meta($user, 'bm_employee', true);

    $query = [];
    $query_key = '';

    switch ($post_type) {
        case 'bm-project':
            $query['key'] = '_bm_project_assigned_to';
            break;
    }

    if (isset($query['key']) && isset($user_bm_employee) && !empty($user_bm_employee)) {
        $query['value'] = $user_bm_employee;
    }

    $args = [
        'numberposts' => -1,
        'post_status' => 'publish',
        'post_type' => $post_type,
        'meta_query' => [
            $query,
        ],
    ];

    $posts = count(get_posts($args));
    $obj = get_post_type_object($post_type);
    $text = sprintf(_n('%s '.$obj->labels->singular_name, '%s '.$obj->labels->name, '<span>'.$posts.'</span>', 'business-manager'), '<span>'.$posts.'</span>');

    return $text;
}

/**
 * Outputs data for the upcoming leave on the dashboard for all employees.
 * @return html
 */
function business_manager_upcoming_leave_html_dashboard()
{
    $employee = new Business_Manager_Employee();
    return $employee->upcoming_leave_html_dashboard();
}

/**
 * Outputs data for the upcoming birthdays on the dashboard for all employees.
 * @return html
 */
function business_manager_upcoming_birthdays_html_dashboard()
{
    $employee = new Business_Manager_Employee();
    return $employee->upcoming_birthdays_html_dashboard();
}

/**
 * Outputs data for the upcoming deadlines on the dashboard for all projects.
 * @return html
 */
function business_manager_upcoming_deadlines_html_dashboard()
{
    $employee = new Business_Manager_Project();
    return $employee->upcoming_deadlines_html_dashboard();
}

/* ============================= LABEL FUNCTIONS ==============================*/
/* =============================================================================*/
// Single label
function business_manager_label_employee_single()
{
    $option = get_option('business-manager-employee');
    return isset($option['singular']) ? esc_html($option['singular']) : __('Employee', 'business-manager');
}

// Plural label
function business_manager_label_employee_plural()
{
    $option = get_option('business-manager-employee');
    return isset($option['plural']) ? esc_html($option['plural']) : __('Employees', 'business-manager');
}

// Single label
function business_manager_label_review_single()
{
    $option = get_option('business-manager-employee');
    return isset($option['review_singular']) ? esc_html($option['review_singular']) : __('Review', 'business-manager');
}

// Plural label
function business_manager_label_review_plural()
{
    $option = get_option('business-manager-employee');
    return isset($option['review_plural']) ? esc_html($option['review_plural']) : __('Reviews', 'business-manager');
}

// Single label
function business_manager_label_leave_single()
{
    $option = get_option('business-manager-employee');
    return isset($option['leave_singular']) ? esc_html($option['leave_singular']) : __('Leave', 'business-manager');
}

// Plural label
function business_manager_label_leave_plural()
{
    $option = get_option('business-manager-employee');
    return isset($option['leave_plural']) ? esc_html($option['leave_plural']) : __('Leave', 'business-manager');
}

// Single label
function business_manager_label_client_single()
{
    $option = get_option('business-manager-client');
    return isset($option['singular']) ? esc_html($option['singular']) : __('Client', 'business-manager');
}

// Plural label
function business_manager_label_client_plural()
{
    $option = get_option('business-manager-client');
    return isset($option['plural']) ? esc_html($option['plural']) : __('Clients', 'business-manager');
}

// Single label
function business_manager_label_project_single()
{
    $option = get_option('business-manager-project');
    return isset($option['singular']) ? esc_html($option['singular']) : __('Project', 'business-manager');
}

// Plural label
function business_manager_label_project_plural()
{
    $option = get_option('business-manager-project');
    return isset($option['plural']) ? esc_html($option['plural']) : __('Projects', 'business-manager');
}

// Single label
function business_manager_label_document_single()
{
    $option = get_option('business-manager-document');
    return isset($option['singular']) ? esc_html($option['singular']) : __('Document', 'business-manager');
}

// Plural label
function business_manager_label_document_plural()
{
    $option = get_option('business-manager-document');
    return isset($option['plural']) ? esc_html($option['plural']) : __('Documents', 'business-manager');
}

/* ============================= HELPER FUNCTIONS ==============================*/
/* =============================================================================*/

/**
 * Check if a module/post type is enabled, or not.
 *
 * @since  1.0.2
 * @return bool
 */
function business_manager_is_enabled($module)
{
    $option = get_option('business-manager-general');
    $return = true;
    if (isset($option['modules'][$module])) {
        $return = false;
    }
    return $return;
}

/**
 * Retrieve the absolute path to the file upload directory without the trailing slash.
 *
 * @since  1.0
 * @return string $path Absolute path to the upload directory
 */
function business_manager_upload_dir()
{
    $wp_upload_dir = wp_upload_dir();
    wp_mkdir_p($wp_upload_dir['basedir'].'/business-manager');
    $path = $wp_upload_dir['basedir'].'/business-manager';

    return apply_filters('business_manager_upload_dir', $path);
}

function business_manager_money_format($amount)
{
    if (strlen($amount) == 0) {
        return;
    }
    $locale = get_locale();
    $option = get_option('business-manager-payroll');
    $currency = isset($option['currency']) ? $option['currency'] : 'USD';
    $fmt = new NumberFormatter($locale, NumberFormatter::CURRENCY);
    $return = $fmt->formatCurrency($amount, $currency);
    return trim($return);
}

function business_manager_date_format($date)
{
    if (!$date) {
        return;
    }
    $date = date_i18n(get_option('date_format'), $date);
    return $date;
}

function business_manager_option($section, $option = null)
{
    if (!$section) {
        return null;
    }
    $opt = get_option($section);
    $return = isset($opt[$option]) ? $opt[$option] : null;
    return $return;
}

/*
 * Inserts a new key/value before the key in the array.
 *
 * @param $key
 *   The key to insert before.
 * @param $array
 *   An array to insert in to.
 * @param $new_key
 *   The key to insert.
 * @param $new_value
 *   An value to insert.
 *
 * @return
 *   The new array if the key exists, FALSE otherwise.
 *
 * @see array_insert_after()
 */
function business_manager_array_insert_before($key, array &$array, $new_key, $new_value)
{
    if (array_key_exists($key, $array)) {
        $new = [];
        foreach ($array as $k => $value) {
            if ($k === $key) {
                $new[$new_key] = $new_value;
            }
            $new[$k] = $value;
        }
        return $new;
    }
    return false;
}

/*
 * Inserts a new key/value after the key in the array.
 *
 * @param $key
 *   The key to insert after.
 * @param $array
 *   An array to insert in to.
 * @param $new_key
 *   The key to insert.
 * @param $new_value
 *   An value to insert.
 *
 * @return
 *   The new array if the key exists, FALSE otherwise.
 *
 * @see array_insert_before()
 */
function business_manager_array_insert_after($key, array &$array, $new_key, $new_value)
{
    if (array_key_exists($key, $array)) {
        $new = [];
        foreach ($array as $k => $value) {
            $new[$k] = $value;
            if ($k === $key) {
                $new[$new_key] = $new_value;
            }
        }
        return $new;
    }
    return false;
}
