<?php

if (! defined('ABSPATH')) {
    exit;
}

if (! class_exists('Business_Manager_Leave_Metaboxes')) :

/**
 * Business_Manager_Setup Class.
 */
class Business_Manager_Leave_Metaboxes {
    public $post_type = 'bm-leave';
    public $date_format = '';

    public $access_type = 'bm_access_leave';
    public $bm_employee_id;
    public $bm_access;

    private $pre = '_bm_leave_';

    public function __construct() {
        $this->date_format = business_manager_datepicker_format();

        $this->hooks();
    }

    /**
     * Hook in to actions & filters.
     *
     * @since 1.0.0
     */
    public function hooks() {
        add_action('plugins_loaded', [$this, 'bm_pluggable']);

        add_action('cmb2_admin_init', [$this, 'metabox_details']);
    }

    /**
    *
    * Set Business Manager Variables that rely on pluggable functions.
    *
    * @since 1.4.1
    */
    public function bm_pluggable() {
        $this->bm_employee_id = business_manager_employee_id(get_current_user_id());
        $this->bm_access = business_manager_employee_access(get_current_user_id(), $this->access_type);
    }

    public function metabox_details() {
        $id = $this->pre.'details_box';

        $box = new_cmb2_box(
            [
                'id' => $id,
                'title' => '<span class="dashicons dashicons-calendar-alt"></span> '.__('Leave Details', 'business-manager'),
                'object_types' => [$this->post_type],
                'classes' => 'bm',
                'priority' => 'high',
            ]
        );

        $fields = [];

        $fields[] = [
            'name' => __('Employee', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'employee',
            'type' => ($this->bm_access == 'limited' ? 'hidden' : 'select'),
            'default' => $this->bm_employee_id,
            'options_cb' => 'business_manager_dropdown_get_employee',
            'attributes' => [
                'required' => 'required',
            ],
        ];

        $fields[] = [
            'name' => __('Type', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'type',
            'type' => 'select',
            'options_cb' => 'business_manager_dropdown_leave_type',
            'classes' => ($this->bm_access == 'limited' ? 'bm-field-clear' : ''),
            'attributes' => [
                'required' => 'required',
            ],
        ];

        $fields[] = [
            'name' => __('First Day of Leave', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'start',
            'type' => 'text_date_timestamp',
            'date_format' => $this->date_format,
            'attributes' => [
                'required' => 'required',
                'autocomplete' => 'off',
            ],
        ];

        $fields[] = [
            'name' => __('Last Day of Leave', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'end',
            'type' => 'text_date_timestamp',
            'date_format' => $this->date_format,
            'attributes' => [
                'required' => 'required',
                'autocomplete' => 'off',
            ],
        ];

        $fields[] = [
            'name' => __('Status', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'status',
            'type' => ($this->bm_access == 'limited' ? 'hidden' : 'select'),
            'default' => 'Requested',
            'attributes' => [
                'required' => 'required',
            ],
            'options_cb' => 'business_manager_dropdown_leave_status',
        ];

        $fields[] = [
            'name' => __('Total Days', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'total_days',
            'type' => 'text',
            'classes' => 'bm-field-quarter',
            'attributes' => [
                'type' => 'number',
                'step' => 'any',
                'min' => '0',
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }
}

endif;

return new Business_Manager_Leave_Metaboxes();
