<?php

if (! defined('ABSPATH')) {
    exit;
}

if (! class_exists('Business_Manager_Project_Metaboxes', false)) :

/**
 * Business_Manager_Setup Class.
 */
class Business_Manager_Project_Metaboxes {
    public $post_type = 'bm-project';
    public $date_format = '';

    public $access_type = 'bm_access_projects';
    public $bm_employee_id;
    public $bm_access;

    private $pre = '_bm_project_';

    public function __construct() {
        $this->date_format = business_manager_datepicker_format();

        $this->hooks();
    }

    /**
     * Hook in to actions & filters.
     *
     * @since 1.0.0
     */
    public function hooks() {
        add_action('plugins_loaded', [$this, 'bm_pluggable']);

        add_action('cmb2_admin_init', [$this, 'metabox_settings']);
        add_action('cmb2_admin_init', [$this, 'metabox_timeline']);
        add_action('cmb2_admin_init', [$this, 'metabox_tasks']);
        add_action('cmb2_admin_init', [$this, 'metabox_files']);
        add_action('cmb2_admin_init', [$this, 'metabox_notes']);

        add_action('cmb2_admin_init', [$this, 'init_custom_fields']);
        add_action('cmb2_admin_init', [$this, 'init_tasks']);
        // add_action( 'cmb2_admin_init', array( $this, 'metabox_permissions' ) );
    }

    /**
    *
    * Set Business Manager Variables that rely on pluggable functions.
    *
    * @since 1.4.1
    */
    public function bm_pluggable() {
        $this->bm_employee_id = business_manager_employee_id(get_current_user_id());
        $this->bm_access = business_manager_employee_access(get_current_user_id(), $this->access_type);
    }

    public function init_custom_fields() {
        require_once BUSINESSMANAGER_DIR.'includes/class-fields.php';
        Business_Manager_Notes_Field::init_notes();
    }

    public function init_tasks() {
        require_once BUSINESSMANAGER_DIR.'includes/project/tasks/class-setup.php';
    }

    public function metabox_settings() {
        $id = $this->pre.'settings_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-admin-settings"></span> '.__('Settings', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            'priority' => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => __('Assigned To', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'assigned_to',
            'type' => ($this->bm_access == 'limited' ? 'hidden' : 'select'),
            'default' => $this->bm_employee_id,
            'options_cb' => 'business_manager_dropdown_get_employee',
        ];

        if (business_manager_is_enabled('clients')) {
            $fields[] = [
                'name' => __('Client', 'business-manager'),
                'desc' => __('', 'business-manager'),
                'id' => $this->pre.'client',
                'type' => 'select',
                'options_cb' => 'business_manager_dropdown_get_client',
            ];
        }

        $fields[] = [
            'name' => __('Complete', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'complete',
            'type' => 'select',
            'classes' => 'bm-field-third',
            'options_cb' => 'business_manager_dropdown_percentages',
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }

    public function metabox_notes() {
        $id = $this->pre.'notes_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-welcome-write-blog"></span> '.__('Notes', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            'priority' => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => __('Project Notes', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'notes',
            'type' => 'notes',
            'classes' => 'bm-field-full',
            'repeatable' => true,
            'date_format' => $this->date_format,
            'options' => [
                'add_row_text' => __('Add Note', 'business-manager'),
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }

    public function metabox_timeline() {
        $id = $this->pre.'timeline_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-clock"></span> '.__('Timeline', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'classes' => 'bm',
            // 'priority'   => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => __('Start Date', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'start_date',
            'type' => 'text_date_timestamp',
            'date_format' => $this->date_format,
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields[] = [
            'name' => __('End Date', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'end_date',
            'type' => 'text_date_timestamp',
            'date_format' => $this->date_format,
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields[] = [
            'name' => __('Work Days', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'work_days',
            'type' => 'text',
            'classes' => 'bm-field-quarter',
            'attributes' => [
                'type' => 'number',
                'step' => 'any',
                'min' => '0',
            ],
        ];

        $fields[] = [
            'name' => __('Calendar Days', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'calendar_days',
            'type' => 'text',
            'classes' => 'bm-field-quarter',
            'attributes' => [
                'type' => 'number',
                'step' => 'any',
                'min' => '0',
            ],
        ];

        $fields[] = [
            'name' => __('Estimated Hours', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'estimated_hours',
            'type' => 'text',
            'classes' => 'bm-field-quarter',
            'attributes' => [
                'type' => 'number',
                'step' => 'any',
                'min' => '0',
            ],
        ];

        $fields[] = [
            'name' => __('Actual Hours', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'actual_hours',
            'type' => 'text',
            'classes' => 'bm-field-quarter',
            'attributes' => [
                'type' => 'number',
                'step' => 'any',
                'min' => '0',
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }

    public function metabox_tasks() {
        $id = $this->pre.'tasks_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-editor-table"></span> '.__('Tasks', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'classes' => 'bm',
            // 'priority'   => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => '',
            'desc' => '',
            'id' => $this->pre.'tasks',
            'type' => 'title',
            'after' => 'business_manager_project_tasks_html_metabox',
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }

        // $box_options = array(
        // 	'id'            => $id,
        // 	'title'         => '<span class="dashicons dashicons-admin-tools"></span> ' . __( 'Tasks', 'business-manager' ),
        // 	'object_types'  => array( $this->post_type ), // Post type
        // 	'context'    	=> 'normal',
        // 	'classes'    	=> 'bm',
        // 	// 'priority'   => 'high',
        // );

        // // Setup meta box
        // $box = new_cmb2_box( $box_options );

        // // setting tabs
        // $tabs_setting           = array(
        // 	'config' => $box_options,
        // 			'layout' => 'vertical', // Default : horizontal
        // 	'tabs'   => array()
        // );

        // $tabs_setting['tabs'][] = array(
        // 	'id'     	=> 'general',
        // 	'title'  	=> __( 'General', 'business-manager' ),
        // 	'desc' 		=> __( '', 'business-manager' ),
        // 	'fields' 	=> array(
        // 		array(
        // 			'name'    => __( 'Test', 'business-manager' ),
        // 			'id'      => $this->pre . 'general_notes',
        // 			'type'    => 'notes',
        // 			'repeatable'    => true,

        // 		),
        // 		array(
        // 			'name'    => __( 'Test 1', 'business-manager' ),
        // 			'id'      => $this->pre . 'general_notes1',
        // 			'type'    => 'notes',
        // 			'repeatable'    => true,

        // 		)
        // 	)
        // );

        // $tabs_setting['tabs'][] = array(
        // 	'id'     	=> 'contracts',
        // 	'title'  	=> __( 'Contracts', 'business-manager' ),
        // 	'desc' 		=> __( 'Contract documents & agreements', 'business-manager' ),
        // 	'fields' 	=> array(
        // 		array(
        // 			'name'    => __( 'Files', 'business-manager' ),
        // 			'id'      => $this->pre . 'contract_files',
        // 			'type'    => 'file_list',
        // 			'options' => array(
        // 				'url' => false
        // 			)
        // 		)
        // 	)
        // );

        // // set tabs
        // $box->add_field( array(
        // 	'id'   => '__tabs',
        // 	'type' => 'tabs',
        // 	'tabs' => $tabs_setting
        // ) );
    }

    public function metabox_files() {
        $id = $this->pre.'files_box';

        $box_options = [
            'id' => $id,
            'title' => '<span class="dashicons dashicons-portfolio"></span> '.__('Files & Documents', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            // 'priority'   => 'high',
        ];

        // Setup meta box
        $box = new_cmb2_box($box_options);

        // setting tabs
        $tabs_setting = [
            'config' => $box_options,
            //		'layout' => 'vertical', // Default : horizontal
            'tabs' => [],
        ];

        $fields['general'][] = [
            'name' => __('Files', 'business-manager'),
            'id' => $this->pre.'general_files',
            'type' => 'file_list',
            'options' => [
                'url' => false,
            ],
        ];

        $fields['contracts'][] = [
            'name' => __('Files', 'business-manager'),
            'id' => $this->pre.'contract_files',
            'desc' => __('Contract documents & files', 'business-manager'),
            'type' => 'file_list',
            'options' => [
                'url' => false,
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        $tabs_setting['tabs'][] = [
            'id' => 'general',
            'title' => __('General', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'fields' => $fields['general'],
        ];
        $tabs_setting['tabs'][] = [
            'id' => 'contracts',
            'title' => __('Contracts', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'fields' => $fields['contracts'],
        ];

        // set tabs
        $box->add_field([
            'id' => '__tabs',
            'type' => 'tabs',
            'tabs' => $tabs_setting,
        ]);
    }
}

endif;

$business_manager_project_metaboxes = new Business_Manager_Project_Metaboxes();
