<?php

if (! defined('ABSPATH')) {
    exit;
}

if (! class_exists('Business_Manager_Review_Metaboxes', false)) :

/**
 * Business_Manager_Setup Class.
 */
class Business_Manager_Review_Metaboxes {
    public $post_type = 'bm-review';
    public $date_format = '';

    private $pre = '_bm_review_';

    public function __construct() {
        $this->date_format = business_manager_datepicker_format();

        $this->hooks();
    }

    /**
     * Hook in to actions & filters.
     *
     * @since 1.0.0
     */
    public function hooks() {
        add_action('cmb2_admin_init', [$this, 'metabox_details']);
        add_action('cmb2_admin_init', [$this, 'metabox_ratings']);
        add_action('cmb2_admin_init', [$this, 'metabox_summary']);
        add_action('cmb2_admin_init', [$this, 'metabox_goals']);
        add_action('cmb2_admin_init', [$this, 'metabox_files']);
        add_action('cmb2_admin_init', [$this, 'metabox_notes']);

        add_action('cmb2_admin_init', [$this, 'init_custom_fields']);
        // add_action( 'cmb2_admin_init', array( $this, 'metabox_permissions' ) );
    }

    public function init_custom_fields() {
        require_once BUSINESSMANAGER_DIR.'includes/class-fields.php';
        Business_Manager_Ratings_Field::init_ratings();
        Business_Manager_Goal_Field::init_goals();
        Business_Manager_Notes_Field::init_notes();
    }

    public function metabox_details() {
        $id = $this->pre.'details_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-admin-settings"></span> '.__('Details', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            'priority' => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => __('Employee', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'employee',
            'type' => 'select',
            'options_cb' => 'business_manager_dropdown_get_employee',
        ];

        $fields[] = [
            'name' => __('Reviewer', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'reviewer',
            'type' => 'select',
            'options_cb' => 'business_manager_dropdown_get_employee',
        ];

        $fields[] = [
            'name' => __('Date of Review', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'date',
            'type' => 'text_date_timestamp',
            'date_format' => $this->date_format,
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        $fields[] = [
            'name' => __('Next Review', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'next_date',
            'type' => 'text_date_timestamp',
            'date_format' => $this->date_format,
            'attributes' => [
                'autocomplete' => 'off',
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }

    public function metabox_notes() {
        $id = $this->pre.'notes_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-welcome-write-blog"></span> '.__('Notes', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            'priority' => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => __('Review Notes', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'notes',
            'type' => 'notes',
            'classes' => 'bm-field-full',
            'repeatable' => true,
            'date_format' => $this->date_format,
            'options' => [
                'add_row_text' => __('Add Note', 'business-manager'),
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }

    public function metabox_files() {
        $id = $this->pre.'file_list_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-portfolio"></span> '.__('Files', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            'priority' => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => __('Review Files', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'files',
            'type' => 'file_list',
            'options' => [
                'url' => false,
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }

    public function metabox_ratings() {
        $id = $this->pre.'ratings_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-star-filled"></span> '.__('Ratings', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            'priority' => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => __('Ratings', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'ratings',
            'type' => 'ratings',
            'repeatable' => true,
            'classes' => 'bm-repeat',
            'text' => [
                'add_row_text' => __('Add Rating', 'business-manager'),
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }

    public function metabox_summary() {
        $id = $this->pre.'summary_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-format-status"></span> '.__('Summary', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            'priority' => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => __('Strong points', 'business-manager'),
            'desc' => __('What are the employee\'s strongest points?', 'business-manager'),
            'id' => $this->pre.'strong_points',
            'type' => 'textarea',
            'attributes' => [
                'rows' => 6,
            ],
        ];

        $fields[] = [
            'name' => __('Weak points', 'business-manager'),
            'desc' => __('What are the employee\'s weakest points?', 'business-manager'),
            'id' => $this->pre.'weak_points',
            'type' => 'textarea',
            'attributes' => [
                'rows' => 6,
            ],
        ];

        $fields[] = [
            'name' => __('Improvements', 'business-manager'),
            'desc' => __('What can the employee\'s do to be more effective or make improvements?', 'business-manager'),
            'id' => $this->pre.'improvements',
            'type' => 'textarea',
            'attributes' => [
                'rows' => 6,
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }

    public function metabox_goals() {
        $id = $this->pre.'goals_box';

        $box = new_cmb2_box([
            'id' => $id,
            'title' => '<span class="dashicons dashicons-chart-bar"></span> '.__('Goals', 'business-manager'),
            'object_types' => [$this->post_type], // Post type
            'context' => 'normal',
            'classes' => 'bm',
            // 'priority'   => 'high',
        ]);

        $fields = [];

        $fields[] = [
            'name' => __('Goals', 'business-manager'),
            'desc' => __('', 'business-manager'),
            'id' => $this->pre.'goals',
            'type' => 'goals',
            'repeatable' => true,
            'classes' => 'bm-repeat',
            'date_format' => $this->date_format,
            'text' => [
                'add_row_text' => __('Add Goal', 'business-manager'),
            ],
        ];

        // filter the fields
        $fields = apply_filters("business_manager_metabox_{$id}", $fields, $id, $this->post_type);

        // sort numerically
        ksort($fields);

        // loop through ordered fields and add them to the metabox
        if ($fields) {
            foreach ($fields as $key => $value) {
                $fields[$key] = $box->add_field($value);
            }
        }
    }
}

endif;

return new Business_Manager_Review_Metaboxes();
